/* billard3d.c
**
**    drawing all with OpenGL
**    Copyright (C) 2001  Florian Berger
**    Email: harpin_floh@yahoo.de, florian.berger@jk.uni-linz.ac.at
**
**    This program is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License Version 2 as
**    published by the Free Software Foundation;
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program; if not, write to the Free Software
**    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <GL/freeglut.h>
//#include <sys/timeb.h>  /* my_time */
#include <sys/time.h>   // us time measure
#include <time.h>   // us time measure
#define _GNU_SOURCE   // getopt_long_only
#include <getopt.h>   // getopt_long_only
#include <dirent.h>   // scandir for mapfiles
#include "globe.h"
#include "png_loader.h"
#include "sunpos.h"
#include "options.h"

#ifdef XMESA
#include <GL/xmesa.h>
#endif


#define LIT 1
#define TEXTURED 2
#define REFLECT 3
#define ANIMATE 10
#define POINT_FILTER 20
#define LINEAR_FILTER 21
#define QUIT 100

//#define WIDTH   1280
//#define HEIGHT  1024
#define WIDTH   1024
#define HEIGHT   768
//#define WIDTH    800
//#define HEIGHT   600


static GLboolean Animate = GL_FALSE;

static GLfloat Xrot = 0.0, Yrot = 0.0, Zrot = 0.0;
static GLfloat DXrot = 0.0, DZrot = 0.0;
static GLfloat scale = 1.0;



int moving, start_x, start_y;
int scaling, scaling_start;
GLfloat cam_dist = 1.2;

static int  key_modifiers;
static int  queue_view=1;
static int  queue_anim=0;

static int  show_fps=0;

static int  stereo_rg_on=0;
static int  show_grid=1;
static int  show_markers=1;
static int  show_dayline=1;
static int  show_date_time=1;
static int  markers_bitmap_mode=0;
static time_t toffs=0;
static time_t tact=0;
static int  tanim=1;
static int  toffs_start=0;
static char mapfilename[80];
static char markerfilename[80];
static int  gridnr_lat=12;
static int  gridnr_long=24;
static int  full_grid=0;
static unsigned  grid_col    = 0xA0FFFFFF;
static unsigned  dayline_col = 0x5000B200;
static unsigned  markers_col = 0x000000FF;
static unsigned  label_col   = 0x00FFFFFF;


static char map_path[256];
static char * map_path_default = "/usr/share/glglobe/";
//static char * map_path = "./";
static struct dirent ** map_namelist;


#define KEY_ACT_TIME        '0'
#define KEY_BITMAP_MARKERS  't'
#define KEY_ZOOM_IN         '+'
#define KEY_ZOOM_OUT        '-'
#define KEY_ANIM_TIME       'a'
#define KEY_ANIM_GLOBE      'r'
#define KEY_FULL_GRID       'f'
#define KEY_GRID_ON_OFF     'g'
#define KEY_MARKERS_ON_OFF  'm'
#define KEY_DAYLINE_ON_OFF  'd'
#define KEY_INFO_ON_OFF     'i'
#define KEY_BENCHMARK       'b'


enum{
    MENU_MARKERS_ON_OFF,
    MENU_GRID_ON_OFF,
    MENU_DAYLINE_ON_OFF,
    MENU_INFO_ON_OFF,
    MENU_FULL_GRID,
    MENU_BITMAP_MARKERS,
    MENU_ACT_TIME,
    MENU_ANIM_TIME,
    MENU_ANIM_GLOBE
};


enum optionType
{
    OPT_MAPFILE,
    OPT_MAPPATH,
    OPT_MARKERFILE,
    OPT_DT,
    OPT_TIME,
    OPT_TANIM,
    OPT_NOTANIM,
    OPT_ANIM,
    OPT_NOANIM,
    OPT_GRID,
    OPT_NOGRID,
    OPT_FULLGRID,
    OPT_SIMPLEGRID,
    OPT_GRIDLAT,
    OPT_GRIDLONG,
    OPT_GRIDCOL,
    OPT_MARKERS,
    OPT_NOMARKERS,
    OPT_MARKERSCOL,
    OPT_LABELCOL,
    OPT_DAYLINE,
    OPT_NODAYLINE,
    OPT_DAYLINECOL,
    OPT_INFO,
    OPT_NOINFO,
    OPT_HELP,
    OPT_DUMMY
};

static struct option long_options[] =
{
    {"mapfile",      required_argument, NULL, OPT_MAPFILE},
    {"mappath",      required_argument, NULL, OPT_MAPPATH},
    {"markerfile",   required_argument, NULL, OPT_MARKERFILE},
    {"dt",           required_argument, NULL, OPT_DT},
    {"time",         required_argument, NULL, OPT_TIME},
    {"tanim",        no_argument,       NULL, OPT_TANIM},
    {"notanim",      no_argument,       NULL, OPT_NOTANIM},
    {"anim",         no_argument,       NULL, OPT_ANIM},
    {"noanim",       no_argument,       NULL, OPT_NOANIM},
    {"grid",         no_argument,       NULL, OPT_GRID},
    {"nogrid",       no_argument,       NULL, OPT_NOGRID},
    {"fullgrid",     no_argument,       NULL, OPT_FULLGRID},
    {"simplegrid",   no_argument,       NULL, OPT_SIMPLEGRID},
    {"gridlat",      required_argument, NULL, OPT_GRIDLAT},
    {"gridlong",     required_argument, NULL, OPT_GRIDLONG},
    {"gridcol",      required_argument, NULL, OPT_GRIDCOL},
    {"markers",      no_argument,       NULL, OPT_MARKERS},
    {"nomarkers",    no_argument,       NULL, OPT_NOMARKERS},
    {"markerscol",   required_argument, NULL, OPT_MARKERSCOL},
    {"labelcol",     required_argument, NULL, OPT_LABELCOL},
    {"dayline",      no_argument,       NULL, OPT_DAYLINE},
    {"nodayline",    no_argument,       NULL, OPT_NODAYLINE},
    {"daylinecol",   required_argument, NULL, OPT_DAYLINECOL},
    {"info",         no_argument,       NULL, OPT_INFO},
    {"noinfo",       no_argument,       NULL, OPT_NOINFO},
    {"help",         no_argument,       NULL, OPT_HELP},
    {NULL,           0,                 NULL, 0}
};




/*
int my_time(void)
{struct timeb ts;
      ftime(&ts);
      return(ts.time*1000+ts.millitm);
}*/


int time_us()
// gives back time in microseconds
{
    struct timeval tv;
    struct timezone tz;
    tz.tz_minuteswest = 0;
    tz.tz_dsttime     = 0;
    gettimeofday(&tv,&tz);
    return ( tv.tv_sec*1000000+tv.tv_usec );
}


static void Idle( void ) /*FOLD00*/
{
   if (Animate) {
       Xrot += DXrot;
       Zrot += DZrot;
       glutPostRedisplay();
   } else {
       glutPostRedisplay();
   }
}




static void
MouseEvent(int button, int state, int x, int y)
{
    key_modifiers=glutGetModifiers();
    if (button == GLUT_LEFT_BUTTON) {
        if (state == GLUT_DOWN) {
            moving = 1;
            start_x = x;
            start_y = y;
        }
        if (state == GLUT_UP) {
            moving = 0;
        }
    }
    if (button == GLUT_RIGHT_BUTTON){
        if (state == GLUT_DOWN) {
            scaling = 1;
            scaling_start = y;
            toffs_start = x;
        }
        if (state == GLUT_UP) {
            scaling = 0;
        }
    }
    if (button == GLUT_MIDDLE_BUTTON) {
        if(!queue_view){
            queue_anim=30;
        } else {
        }
    }
    glutPostRedisplay();
}


static void
MouseMotion(int x, int y)
{
    static double acc;

    if(moving && scaling){
//        cam_look_offs = vec_add(cam_look_offs, )
    } else if(moving){
        if( key_modifiers & GLUT_ACTIVE_CTRL ) acc=0.02; else acc=1.0;
        if(!Animate){
            Xrot +=  (double)(y-start_y)*acc;
            Zrot +=  (double)(x-start_x)*acc;
        } else {
            DZrot = (double)(x-start_x)*0.05;
            DXrot = (double)(y-start_y)*0.05;
        }
        start_x = x;
        start_y = y;
    } else if(scaling){
        if (abs(y-scaling_start)>abs(x-toffs_start)){
            cam_dist +=  (y-scaling_start)*0.001;
            if( cam_dist<0.0 ) cam_dist=0.0;
            scaling_start = y;
            toffs_start = x;
        } else {
            toffs += (x-toffs_start)*60;
            scaling_start = y;
            toffs_start = x;
        }
    }
    glutPostRedisplay();
}


static void DisplayFunc( void )
{
//   int i;
   double th,ph;
   GLfloat light_position[] = { 0.0, 0.0, 180.0, 0.0 };
   GLfloat light_diff[] = { 1.0, 1.0, 1.0, 1.0 };
   GLfloat light_amb[] = { 0.5, 0.5, 0.5, 1.0 };
   GLfloat light_diff_r[] = { 1.0, 0.0, 0.0, 1.0 };
   GLfloat light_amb_r[] = { 0.5, 0.0, 0.0, 1.0 };
   GLfloat light_diff_g[] = { 0.0, 0.7, 0.0, 1.0 };
   GLfloat light_amb_g[] = { 0.0, 0.5*0.7, 0.0, 1.0 };

   VMvect cam_pos;
   static GLfloat real_dist=0.0;
   int t_act;
   static int dt;
   static int t_prev;
   static int count=0;
   int i;
   int act_buffer;
   VMvect light_pos;
   VMvect light_pos2;
   struct tm * tmstruct;

   static GLfloat rg_eye_dist=0.06;
   static GLfloat rg_scrw=0.225;
   static GLfloat rg_scrh=0.165;
   time_t t_total;


   if (tanim){
       tact=time(NULL);
   }

   t_total=tact+toffs;
   tmstruct=localtime(&t_total);
   if( tmstruct->tm_isdst ) t_total-=3600;

//   tact=913456789-3600;
//   toffs+=500;
   sunpos(t_total,&light_pos,1);
   sunpos(t_total,&light_pos2,0);
   light_position[0]=light_pos.x;
   light_position[1]=light_pos.y;
   light_position[2]=light_pos.z;

   if(show_fps){
       count++;
       t_act  = time_us();
       dt     += t_act-t_prev;
       t_prev = t_act;
       if(count==5){
           count=0;
           fprintf(stderr,"fps=%f\n",5.0/(double)dt*1000000.0);
           dt=0;
       }
   }

   real_dist = (real_dist*0.94)+(cam_dist+vec_abs(vec_xyz(0,0,0))*0.4)*0.06;
   glGetIntegerv(GL_DRAW_BUFFER, &act_buffer);
   for(i=0;i<=stereo_rg_on;i++){
   glMatrixMode( GL_PROJECTION );
   if       (i==0 && stereo_rg_on){
       glDrawBuffer(GL_AUX0);
       glReadBuffer(GL_AUX0);
       glLoadIdentity();
       glFrustum( (-rg_scrw-rg_eye_dist)/2.0, (rg_scrw-rg_eye_dist)/2.0,
                  -rg_scrh/2.0, rg_scrh/2.0, 0.30, 5.0);
   }else if (i==1 && stereo_rg_on){
       glDrawBuffer(GL_AUX1);
       glReadBuffer(GL_AUX1);
       glLoadIdentity();
       glFrustum( (-rg_scrw+rg_eye_dist)/2.0, (rg_scrw+rg_eye_dist)/2.0,
                  -rg_scrh/2.0, rg_scrh/2.0, 0.30, 5.0);
   }else{
//       glFrustum( -0.1, 0.1, -0.08, 0.08, 0.30, 5.0);
   }
   glMatrixMode( GL_MODELVIEW );
   glLoadIdentity();

   if       (i==0 && stereo_rg_on){
       glTranslatef( -rg_eye_dist/2.0, 0.0, 0.0 );
   }else if (i==1 && stereo_rg_on){
       glTranslatef( rg_eye_dist/2.0, 0.0, 0.0 );
   }else{
   }
       glTranslatef( 0.0, 0.0, -real_dist );

   if      (i==0 && stereo_rg_on){
       glLightfv(GL_LIGHT0, GL_DIFFUSE,  light_diff_g);
       glLightfv(GL_LIGHT0, GL_AMBIENT,  light_amb_g);
       glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
   }else if(i==1 && stereo_rg_on){
       glLightfv(GL_LIGHT0, GL_DIFFUSE,  light_diff_r);
       glLightfv(GL_LIGHT0, GL_AMBIENT,  light_amb_r);
       glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
//       glEnable(GL_BLEND);
//       glBlendFunc (GL_ONE, GL_ONE);
   } else {
       glLightfv(GL_LIGHT0, GL_DIFFUSE,  light_diff);
       glLightfv(GL_LIGHT0, GL_AMBIENT,  light_amb);
       glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
//       glMaterialfv(GL_FRONT, GL_DIFFUSE,  col_black);
//       glRectf( -1.0,-1.0,1.0,1.0 );
//       glClear( GL_DEPTH_BUFFER_BIT );
   }

   glPushMatrix();

   glRotatef(Xrot, 1.0, 0.0, 0.0);
   glRotatef(Yrot, 0.0, 1.0, 0.0);
   glRotatef(Zrot, 0.0, 0.0, 1.0);

   glLightfv(GL_LIGHT0, GL_POSITION, light_position);

   /* draw table */
//   glCallList(table_obj);

//   glScalef(100.0,100.0,100.0);

   th=Xrot/180.0*M_PI;
   ph=Zrot/180.0*M_PI;
   cam_pos=vec_scale(vec_xyz(sin(th)*sin(ph),sin(th)*cos(ph),cos(th)),
                     real_dist);
//   my_glBox(cam_pos.x+0.01, cam_pos.y+0.01, cam_pos.z+0.01,
//            cam_pos.x-0.01, cam_pos.y-0.01, cam_pos.z-0.01);


   glPushMatrix();
   draw_globe(mapfilename,map_path);
   if(show_dayline){
//       draw_dayline(light_pos2, 0.2,0.3,0.2);
       draw_dayline(light_pos, dayline_col
/*                    (double)((dayline_col>>16) & 0xFF)/255.0,
                    (double)((dayline_col>>8 ) & 0xFF)/255.0,
                    (double)((dayline_col>>0 ) & 0xFF)/255.0,
                    1.0-(double)((dayline_col>>24 ) & 0xFF)/255.0*/
                   );
   }
   if(show_grid){
       if (full_grid){
           draw_grid( gridnr_long, gridnr_lat, grid_col );
       } else {
           draw_grid_marks( gridnr_long, gridnr_lat, grid_col );
       }
   }
   if(show_markers){
       draw_markers( markerfilename, markers_bitmap_mode,
                     markers_col, label_col );
   }
   if(show_date_time){
       char str[200];
       double angN, angW;
//       t_total=898434180+60*57+24*3600*100;
       angW=atan2(light_pos.y,-light_pos.x)*180.0/M_PI;
       angN=atan(light_pos.z/sqrt(light_pos.x*light_pos.x+light_pos.y*light_pos.y))*180.0/M_PI;
//       t_total=998712500;
       sprintf(str,"%s - %fN/%fW  %ld",ctime(&t_total),angN,angW,t_total);
//       fprintf(stderr,"date:  %s\n",str);
       glPushMatrix();
       glLoadIdentity();
       glColor3f(0.5,1.0,0.7);
       glDisable(GL_LIGHTING);
       glDisable(GL_TEXTURE_2D);
       glDisable(GL_DEPTH_TEST);
/*       myGlWrite( vec_xyz(-0.30,-0.25,-1.2),
                  vec_ex(), vec_ey(), 1, 0.007, 1, str );*/
       glRasterPos3f(-0.35,-0.27,-1.2);
       for(i=0;str[i]!=0;i++){
            glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18,str[i]);
//            glutBitmapCharacter(GLUT_BITMAP_8_BY_13,str[i]);
       }

       glEnable(GL_DEPTH_TEST);
       glEnable(GL_LIGHTING);
       glEnable(GL_TEXTURE_2D);
       glPopMatrix();
   }
   glPopMatrix();

   /* draw balls with reflections and shadows */
//   draw_balls(balls,cam_pos,spheretexbind);

   glPopMatrix();

   } // for RG-frames

/*   if(stereo_rg_on) {
       glDrawBuffer(act_buffer);
       glReadBuffer(GL_AUX1);
       glCopyPixels(0,0,WIDTH,HEIGHT,GL_COLOR);
   }*/

   glutSwapBuffers();
}


static void Reshape( int width, int height ) /*FOLD00*/
{
/*   if (width > WIDTH)
       width = WIDTH;
   if (height > HEIGHT)
       height = HEIGHT;*/

   glViewport( 0, 0, width, height );
   glMatrixMode( GL_PROJECTION );
   glLoadIdentity();
//   glFrustum( -1.0, 1.0, -1.0, 1.0, 4.0, 1000.0 );
   gluPerspective(27.0, /* field of view in degree */
     (GLfloat) width/(GLfloat) height, /* aspect ratio */
     0.01, /* Z near */
     10.0); /* Z far */
//   gluLookAt(0.0, 1.0, 380.0,  /* eye is at (0,8,60) */
//     0.0, 0.0, 0.0,      /* center is at (0,8,0) */
//     0.0, 1.0, 0.0);      /* up is in postivie Y direction */
}


static void SetMode(GLuint m) /*FOLD00*/
{
   /* disable everything */
   glDisable(GL_LIGHTING);
   glDisable(GL_TEXTURE_2D);
   glDisable(GL_TEXTURE_GEN_S);
   glDisable(GL_TEXTURE_GEN_T);

   /* enable what's needed */
   if (m==LIT) {
      glEnable(GL_LIGHTING);
   }
   else if (m==TEXTURED) {
      glEnable(GL_LIGHTING);
      glEnable(GL_TEXTURE_2D);
   }
   else if (m==REFLECT) {
      glEnable(GL_LIGHTING);
      glEnable(GL_TEXTURE_2D);
      glEnable(GL_TEXTURE_GEN_S);
      glEnable(GL_TEXTURE_GEN_T);
   }
}


static void MapMenu(int entry)
{
    strcpy(mapfilename,map_path);
    strcat(mapfilename,map_namelist[entry]->d_name);
}


static void MainMenu(int entry) /*FOLD00*/
{
    if        (entry==MENU_MARKERS_ON_OFF) {
        show_markers = (show_markers==0)?1:0;
    } else if (entry==MENU_GRID_ON_OFF) {
        show_grid = (show_grid==0)?1:0;
    } else if (entry==MENU_DAYLINE_ON_OFF) {
        show_dayline = (show_dayline==0)?1:0;
    } else if (entry==MENU_INFO_ON_OFF) {
        show_date_time = (show_date_time==0)?1:0;
    } else if (entry==MENU_FULL_GRID) {
        full_grid = (full_grid==0)?1:0;
    } else if (entry==MENU_BITMAP_MARKERS) {
        markers_bitmap_mode = !markers_bitmap_mode;
    } else if (entry==MENU_ACT_TIME) {
        toffs = 0;
    } else if (entry==MENU_ANIM_TIME) {
        tanim = !tanim;
    } else if (entry==MENU_ANIM_GLOBE) {
        Animate = !Animate;
    } else if (entry==QUIT) {
        exit(0);
    } else {
        SetMode(entry);
    }
    glutPostRedisplay();
}


static void Key( unsigned char key, int x, int y ) /*FOLD00*/
{
   switch (key) {
      case 27:
         exit(0);
         break;
      case KEY_ACT_TIME:
           toffs = 0;
         break;
      case KEY_BITMAP_MARKERS:
           markers_bitmap_mode = !markers_bitmap_mode;
         break;
      case KEY_ZOOM_IN:
          cam_dist /= 1.1;
         break;
      case KEY_ZOOM_OUT:
          cam_dist *= 1.1;
         break;
      case KEY_ANIM_TIME:
          tanim = (tanim==0)?1:0;
         break;
      case KEY_FULL_GRID:
          full_grid = (full_grid==0)?1:0;
         break;
      case KEY_GRID_ON_OFF:
          show_grid = (show_grid==0)?1:0;
         break;
      case KEY_MARKERS_ON_OFF:
          show_markers = (show_markers==0)?1:0;
         break;
      case KEY_DAYLINE_ON_OFF:
          show_dayline = (show_dayline==0)?1:0;
         break;
      case KEY_INFO_ON_OFF:
          show_date_time = (show_date_time==0)?1:0;
         break;
      case KEY_BENCHMARK:
          show_fps = (show_fps==0)?1:0;
         break;
      case KEY_ANIM_GLOBE:
          Animate = !Animate;
         break;
      case 's':
          stereo_rg_on = (stereo_rg_on==0)?1:0;
         break;
#ifdef XMESA
       case 'f':
           fullscreen=(!fullscreen);
           XMesaSetFXmode(fullscreen ? XMESA_FX_FULLSCREEN : XMESA_FX_WINDOW);
           break;
#endif
   }
   glutPostRedisplay();
}


static void SpecialKey( int key, int x, int y ) /*FOLD00*/
{
   float step = 3.0;

   switch (key) {
      case GLUT_KEY_UP:
         Xrot += step;
         break;
      case GLUT_KEY_DOWN:
         Xrot -= step;
         break;
      case GLUT_KEY_LEFT:
         Zrot += step;
         break;
      case GLUT_KEY_RIGHT:
         Zrot -= step;
         break;
      case GLUT_KEY_PAGE_UP:
         Yrot += step;
         break;
      case GLUT_KEY_PAGE_DOWN:
         Yrot -= step;
         break;
      case GLUT_KEY_HOME:
         toffs -= 500;
         break;
      case GLUT_KEY_END:
         toffs += 500;
         break;
      case GLUT_KEY_F2:
         toffs += 24.0*3600.0*1.0;
         break;
      case GLUT_KEY_F1:
         toffs -= 24.0*3600.0*1.0;
         break;
      case 'x':
         scale *= 1.1;
         break;
      case 'y':
         scale /= 1.1;
         break;
   }
   glutPostRedisplay();
}


static void Init( void ) /*FOLD00*/
{
//    int i;

    if ( options_dither )
        glEnable(GL_DITHER);
    else
        glDisable(GL_DITHER);

    glEnable(GL_TEXTURE_2D);


    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
//    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);


//   glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
//   glShadeModel (GL_FLAT);

    /* lighting */
    glEnable(GL_LIGHTING);
    {
       GLfloat table_col[4] = { 0.05, 0.5, 0.2, 1.0 };
       GLfloat white[4] = {1.0, 1.0, 1.0, 1.0};
       GLfloat amb[4] =  {0.4, 0.4, 0.4, 1.0};
       GLfloat light_position[] = { 0.0, 300.0, 0.0, 0.0 };
       glMaterialfv(GL_FRONT, GL_DIFFUSE, table_col);
       glLightfv(GL_LIGHT0, GL_POSITION, light_position);
       glLightfv(GL_LIGHT0, GL_AMBIENT, amb);
       glLightfv(GL_LIGHT0, GL_DIFFUSE, white);
       glEnable(GL_LIGHT0);
    }


   glEnable(GL_DEPTH_TEST);
   glFrontFace(GL_CW);
//   glCullFace(GL_FRONT);
   glEnable(GL_CULL_FACE);
   glDepthMask(GL_TRUE);
//   glDepthFunc( GL_GREATER );
//   glDepthFunc( GL_LESS );
   glDepthFunc( GL_LEQUAL );
//   glDepthFunc( GL_ALWAYS );
//   glDepthFunc( GL_EQUAL );

   SetMode(REFLECT);
}


void print_help(struct option * opt)
{
    int i;

    printf("usage: glglobe [--option [<arg>]]\n");
    printf("  options:\n");
    for(i=0;opt[i].name!=0;i++){
        switch(opt[i].val){
        case OPT_MAPFILE:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     specify mapfile (jpg or png)\n");
            break;
        case OPT_MAPPATH:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     path to map-directory\n");
            break;
        case OPT_MARKERFILE:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     specify marker file (like for xglobe)\n");
            break;
        case OPT_DT:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     specify time-offset in seconds\n");
            break;
        case OPT_TIME:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     specify absolute time in seconds (since begin of unix period)\n");
            break;
        case OPT_TANIM:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     turn time animation on\n");
            break;
        case OPT_NOTANIM:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     turn time animation off\n");
            break;
        case OPT_ANIM:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     turn globe animation on\n");
            break;
        case OPT_NOANIM:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     turn globe animation off\n");
            break;
        case OPT_GRID:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     show grid\n");
            break;
        case OPT_NOGRID:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     don't show grid\n");
            break;
        case OPT_FULLGRID:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     show full grid\n");
            break;
        case OPT_SIMPLEGRID:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     show only grid-marks\n");
            break;
        case OPT_GRIDLAT:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     specify number of latitudes\n");
            break;
        case OPT_GRIDLONG:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     specify number of longitudes\n");
            break;
        case OPT_GRIDCOL:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     color of grid in C-style hex notation <0xttbbggrr>\n");
            break;
        case OPT_MARKERS:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     turn markers on\n");
            break;
        case OPT_NOMARKERS:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     turn markers off\n");
            break;
        case OPT_MARKERSCOL:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     color of markers in C-style hex notation <0xbbggrr>\n");
            break;
        case OPT_LABELCOL:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     color of marker-labels in C-style hex notation <0xbbggrr>\n");
            break;
        case OPT_DAYLINE:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     turn on dayline\n");
            break;
        case OPT_NODAYLINE:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     turn off dayline\n");
            break;
        case OPT_DAYLINECOL:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     color of dayline in C-style hex notation <0xttbbggrr>\n");
            break;
        case OPT_INFO:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     show time/date info\n");
            break;
        case OPT_NOINFO:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     don't show time/date info\n");
            break;
        case OPT_HELP:
            printf("--%s %s\n",opt[i].name,opt[i].has_arg?"<arg>":"");
            printf("     show this help\n");
            break;
        }
    }
    printf("the color <0xttbbggrr> means one byte for each transparency, blue, green, red\n");
    printf("the transparency specification is optional e.g. <0xbbggrr>\n");
    printf("\n");
}



int is_map(const struct dirent * entry)
{
    int rval=1;
    if( entry->d_type == DT_DIR ) rval=0;
    if( strcasecmp(&entry->d_name[strlen(entry->d_name)-4],".png") !=0 &&
        strcasecmp(&entry->d_name[strlen(entry->d_name)-4],".jpg") !=0 )
        rval=0;
    return rval;
}



int main( int argc, char *argv[] ) /*FOLD00*/
{

    int auxnr;
    int act_option,option_index;
    char str[80];

    int MapMenuID, MainMenuID;

/*    char * helptext[]=
    {
        "display map"
    }*/

   /* Tell Mesa GLX to use 3Dfx driver in fullscreen mode. */
//   putenv("MESA_GLX_FX=fullscreen");

   /* Disable 3Dfx Glide splash screen */
//   putenv("FX_GLIDE_NO_SPLASH=");


   strcpy(map_path, map_path_default);
//   strcpy(mapfilename,map_path);
   strcpy(mapfilename, map_path);
   strcat(mapfilename,"depths_1440.jpg");
//   strcpy(mapfilename,"depths_1440.jpg");
   strcpy(markerfilename, map_path);
   strcat(markerfilename,"xglobe-markers");

   tact=time(NULL);

   while((act_option = getopt_long_only(argc, argv, "+", long_options,
                                        &option_index)) >= 0){
       switch(act_option){
       case OPT_MAPFILE:   strcpy(mapfilename,optarg); break;
       case OPT_MAPPATH:   strcpy(map_path,optarg); break;
       case OPT_MARKERFILE:strcpy(markerfilename,optarg); break;
       case OPT_DT:        sscanf(optarg,"%ld",&toffs); break;
       case OPT_TIME:      sscanf(optarg,"%ld",&toffs); toffs-=tact; break;
       case OPT_TANIM:     tanim=1; break;
       case OPT_NOTANIM:   tanim=0; break;
       case OPT_ANIM:      Animate=1; break;
       case OPT_NOANIM:    Animate=0; break;
       case OPT_GRID:      show_grid=1; break;
       case OPT_NOGRID:    show_grid=0; break;
       case OPT_FULLGRID:  full_grid=1; break;
       case OPT_SIMPLEGRID:full_grid=0; break;
       case OPT_GRIDLAT:
           sscanf(optarg,"%d",&gridnr_lat);
//           fprintf(stderr,"OPTION - gridnr_lat: <%d>\n",gridnr_lat);
           break;
       case OPT_GRIDLONG:
           sscanf(optarg,"%d",&gridnr_long);
//           fprintf(stderr,"OPTION - gridnr_long: <%d>\n",gridnr_long);
           break;
       case OPT_GRIDCOL:   sscanf(optarg,"%x",&grid_col); break;
       case OPT_MARKERS:   show_markers=1;   break;
       case OPT_NOMARKERS: show_markers=0;   break;
       case OPT_MARKERSCOL:sscanf(optarg,"%x",&markers_col); break;
       case OPT_LABELCOL:  sscanf(optarg,"%x",&label_col); break;
       case OPT_DAYLINE:   show_dayline=1;   break;
       case OPT_NODAYLINE: show_dayline=0;   break;
       case OPT_DAYLINECOL:sscanf(optarg,"%x",&dayline_col); break;
       case OPT_INFO:      show_date_time=1; break;
       case OPT_NOINFO:    show_date_time=0; break;
       case OPT_HELP:      print_help(long_options); exit(1);   break;
       }
   }

   printf("map_path=%s\n",map_path);
   printf("mapfilename=%s\n",mapfilename);
   printf("markerfilename=%s\n",markerfilename);

   glutInitWindowPosition(0, 0);
   //   glutInitWindowSize( 800, 800 );
   glutInitWindowSize(WIDTH, HEIGHT);

   glutInit( &argc, argv );

   glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH );
   //   glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE );

   glutCreateWindow(argv[0] );

/*   if( argc > 1 ){
       strcpy(mapfilename,argv[1]);
   } else {
       strcpy(mapfilename,"depths_1536.png");
   }
   fprintf(stderr,"MAPFILENAME=%s\n",mapfilename);*/

//   glutFullScreen();

   Init();

   glGetIntegerv(GL_AUX_BUFFERS, &auxnr);
#ifdef DEBUG
   fprintf(stderr,"# of AUX-buffers:%d\n",auxnr);
#endif

   glEnable(GL_LIGHTING);
   glutReshapeFunc( Reshape );
   glutKeyboardFunc( Key );
   glutSpecialFunc( SpecialKey );
   glutDisplayFunc( DisplayFunc );
   glutIdleFunc( Idle );
   glutMouseFunc(MouseEvent);
//   glutPassiveMotionFunc( MouseMotion );
   glutMotionFunc(MouseMotion);


   MapMenuID=glutCreateMenu(MapMenu);
   {
       int n,i;
       n=scandir(map_path,&map_namelist,is_map,alphasort);
       for(i=0;i<n;i++){
           glutAddMenuEntry( map_namelist[i]->d_name, i );
       }
   }

   MainMenuID=glutCreateMenu(MainMenu);
/*   glutAddMenuEntry("Lit", LIT);
   glutAddMenuEntry("Textured", TEXTURED);
   glutAddMenuEntry("Reflect", REFLECT);
   glutAddMenuEntry("Point Filtered", POINT_FILTER);
   glutAddMenuEntry("Linear Filtered", LINEAR_FILTER);
   glutAddMenuEntry("Toggle Animation", ANIMATE);*/
   sprintf(str,"Markers ON/OFF   <%c>",  KEY_MARKERS_ON_OFF);
   glutAddMenuEntry(str,  MENU_MARKERS_ON_OFF );
   sprintf(str,"Grid    ON/OFF   <%c>",  KEY_GRID_ON_OFF);
   glutAddMenuEntry(str,  MENU_GRID_ON_OFF);
   sprintf(str,"Dayline ON/OFF   <%c>",  KEY_DAYLINE_ON_OFF);
   glutAddMenuEntry(str,  MENU_DAYLINE_ON_OFF);
   sprintf(str,"Info    ON/OFF   <%c>",  KEY_INFO_ON_OFF);
   glutAddMenuEntry(str,  MENU_INFO_ON_OFF);
   sprintf(str,"Full/Simple Grid <%c>",  KEY_FULL_GRID);
   glutAddMenuEntry(str,  MENU_FULL_GRID);
   sprintf(str,"Bitmap Markers   <%c>",  KEY_BITMAP_MARKERS);
   glutAddMenuEntry(str,  MENU_BITMAP_MARKERS);
   sprintf(str,"Set actual time  <%c>",  KEY_ACT_TIME);
   glutAddMenuEntry(str,  MENU_ACT_TIME);
   sprintf(str,"start/stop time  <%c>",  KEY_ANIM_TIME);
   glutAddMenuEntry(str,  MENU_ANIM_TIME);
   sprintf(str,"start/stop animation  <%c>",  KEY_ANIM_GLOBE);
   glutAddMenuEntry(str,  MENU_ANIM_GLOBE);
   sprintf(str,"Quit   <ESC>");
   glutAddMenuEntry(str, QUIT);
   glutAddSubMenu("Maps", MapMenuID);
   glutAttachMenu(GLUT_MIDDLE_BUTTON);

   glutMainLoop();
   return 0;
}
